// Some book references
// 
// 1. Bjarne Strousstrup, The C++ Programming Language.
//
// 2. Andrea Domenici, Graziano Frosini, Introduzione alla Programmazione ed
//    Elementi di Strutture Dati con il Linguaggio C++.

#ifndef __VECTOR_HPP__
#define __VECTOR_HPP__

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

class Vector
{
// the private part contains data and methods
// not accessible from outside and hidden to the
// children
private:
    float *storage;

// the protected part contains data and methods
// not accessible from outside but that can be
// accessed by children
protected:
    unsigned int N;

    // method to deallocate preexisting objects
    void disposeHelper()
    {
        delete[] storage;   // delete operator substitute free()
    }

    void copyHelper(const Vector &v)
    {
        N=v.getN();

        if (N)
            storage=new float[N];

        for (unsigned int i=0; i<N; i++)
            storage[i]=v.getElem(i);
    }

// the public part contains data and methods
// accessible from outside
public:
    // constructor
    Vector(const unsigned int nelem)
    {
        fprintf(stdout,"Vector class: calling parameters_list-constructor\n");

        N=nelem;

        if (N)
            storage=new float[N];   // new operator substitute malloc()
    }

    // copy constructor
    Vector(const Vector &v)
    {
        fprintf(stdout,"Vector class: calling copy-constructor\n");

        copyHelper(v);
    }

    // method to make an exctact copy of a given vector
    void copy(const Vector &v)
    {
        disposeHelper();
        copyHelper(v);
    }

    // method to return number of elements
    // const methods are the only ones that
    // can be called when a reference is used
    // [see the copy constructor]
    unsigned int getN() const
    {
        return N;
    }

    // method to access specific element
    // for read/write operations
    float &getElem(const unsigned int i)
    {
        return storage[i];
    }

    // method to access specific element
    // overloaded const version used just for read operations
    float &getElem(const unsigned int i) const
    {
        return storage[i];
    }

    // method to implement vector sum
    Vector add(const Vector &v) const
    {
        // the this operator is a pointer to this particular
        // instance of object
        Vector res=*this; // call the copy constructor

        if (v.getN()==N)
            for (unsigned int i=0; i<N; i++)
                res.getElem(i)=storage[i]+v.getElem(i);

        return res;
    }

    // method to implement vector sub
    Vector sub(const Vector &v) const
    {
        Vector res=*this;

        if (v.getN()==N)
            for (unsigned int i=0; i<N; i++)
                res.getElem(i)=storage[i]-v.getElem(i);

        return res;
    }

    // method to implement dot product
    float dot(const Vector &v) const
    {
        float res=0.0;

        if (v.getN()==N)
            for (unsigned int i=0; i<N; i++)
                res+=storage[i]*v.getElem(i);

        return res;
    }

    // method to implement the euclidean norm
    float norm() const
    {
        return sqrt(dot(*this));
    }

    // method to print vector components
    void print() const
    {
        for (unsigned int i=0; i<N; i++)
            fprintf(stdout,"%d: %.3f\n",i+1,storage[i]);
    }

    // destructor
    ~Vector()
    {
        fprintf(stdout,"Vector class: calling destructor\n");

        disposeHelper();
    }
};

#endif


