// This example shows out the encapsulation concept of C++
// implementing the class Vector in contrast with the procedural
// approach of C.

// The new reference type is also presented.

#include "vector.hpp"

int main()
{
    // make practice with references
    int n=1;
    int &ref=n;

    fprintf(stdout,"print the integer: %d\n",n);
    fprintf(stdout,"print the reference: %d\n",ref);

    // change the integer value and see what happens to the reference
    n+=1;
    fprintf(stdout,"print the reference: %d\n",ref);

    // now change the reference value and see what happens to the integer object
    ref+=1;
    fprintf(stdout,"print the integer: %d\n",n);

    // be careful, a reference must be always initialized
    // int &ref; This is not allowed!!
    // a reference indeed cannot be reassinged as an alias to 
    // another object: we don not have the control on the life of the referenced object

    // *********************************************
    // let's go to the Vector class

    // definitions
    Vector v1(3), v2(3);

    // fill up vectors components
    v1.getElem(0)=1.0F;  v2.getElem(0)=0.0F;
    v1.getElem(1)=2.0F;  v2.getElem(1)=1.0F;
    v1.getElem(2)=3.0F;  v2.getElem(2)=0.0F;

    fprintf(stdout,"first vector components\n");
    v1.print();

    fprintf(stdout,"second vector components\n");
    v2.print();

    // execute algebric operations on vectors
    Vector w=v1.add(v2);
    // we expect a call to the copy-constructor for the res object internal to add()
    // a call to the copy-constructor for w
    // a call to the desctructor to dispose res

    float dot=v1.dot(v2);
    float norm=w.norm();

    // dump results
    fprintf(stdout,"sum\n");
    w.print();

    fprintf(stdout,"dot product\n");
    fprintf(stdout,"%.3f\n",dot);

    fprintf(stdout,"euclidean norm\n");
    fprintf(stdout,"%.3f\n",norm);

    // try out the copy constructor in the very simple form
    Vector z=w;
    fprintf(stdout,"copy vector\n");
    z.print(); 

    // try out the dynamic allocation
    Vector *dyn=new Vector(2);  // take a look to the new operator used to instantiate a class object
    // note: new operator calls the corresponding constructor; malloc() cannot do the same thing!!!
    dyn->getElem(0)=0.1F;
    dyn->getElem(1)=0.2F;
    fprintf(stdout,"dynamic allocated vector\n");
    dyn->print(); 

    // ...and dispose it
    delete dyn; // delete operator calls the destructor

    // here, before exiting the main(), destructors of
    // all automatic objects declared within the current scope
    // will be called

    return 0;
}

