
// This example shows out the polymorphism concept of C++

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

// hereafter we start again from the father class but with a
// subset of methods to make things easier and quicker and
// just to highlight the use of "virtual" attribute
class Vector
{
protected:
    // now storage is protected so that
    // it can be referenced by children classes
    float *storage;

    unsigned int N;

public:
    Vector(const unsigned int nelem)
    {
        N=nelem;

        if (N)
            storage=new float[N];
    }

    float &getElem(const unsigned int i)
    {
        return storage[i];
    }

    // this time the method is virtual, i.e. polymorphic
    virtual float norm() const
    {
        float res=0.0;

        // we do not resot to dot() method
        // since now it is not available
        for (unsigned int i=0; i<N; i++)
            res+=storage[i]*storage[i];

        return sqrt(res);
    }

    void print() const
    {
        for (unsigned int i=0; i<N; i++)
            fprintf(stdout,"%d: %.3f\n",i+1,storage[i]);
    }

    ~Vector()
    {
        delete[] storage;
    }
};


class VectorInf : public Vector
{
public:
    // constructor: just call the father's one
    VectorInf(const unsigned int nelem) : Vector(nelem) { }

    // polymorphic method to implement infinity norm
    virtual float norm() const
    {
        float res=0.0;

        for (unsigned int i=0; i<N; i++)
        {
            float tmp=abs(storage[i]);

            if (tmp>res)
                res=tmp;
        }

        return res;
    }

    // we redefine the print() method in order to 
    // print in one raw, but not using polymorphism
    void print() const
    {
        fprintf(stdout,"1=>%d: ",N);

        for (unsigned int i=0; i<N; i++)
            fprintf(stdout,"%.3f ",storage[i]);

        fprintf(stdout,"\n");
    }

    // also the destructor can be virtualized if needed
};


int main()
{
    // definitions
    Vector    v1(3);
    VectorInf v2(3);

    // fill up vectors components
    v1.getElem(0)=1.0F;  v2.getElem(0)=1.0F;
    v1.getElem(1)=2.0F;  v2.getElem(1)=2.0F;
    v1.getElem(2)=3.0F;  v2.getElem(2)=3.0F;

    // declare a generic pointer to Vector class hierarchy.
    // note: polymorphism can be accomplished only through pointers
    Vector *w;

    w=&v1;  // make w point to v1
    fprintf(stdout,"first vector components\n");
    w->print();
    fprintf(stdout,"norm is %.3f\n",w->norm());

    w=&v2;  // make w point to v2
    fprintf(stdout,"second vector components\n");
    w->print();  // this won't work as expected...
    v2.print(); // ...since it should look like as the output of this
    fprintf(stdout,"norm is %.3f\n",w->norm()); // conversely this is ok thanks to the polymorphism

    return 0;
}

