// Just a C module that implements some basic algebric 
// operations between vectors.

#include <stdlib.h>
#include <stdio.h>

typedef struct
{
    float *storage;
    int N;
} Vector;



void print_vector(const Vector *v)
{
    int i;

    for (i=0; i<v->N; i++)
        fprintf(stdout,"%d: %.3f\n",i+1,v->storage[i]);
}


int add(const Vector *v1, const Vector *v2, Vector *res)
{    
    if (v1->N==v2->N)
    {
        int i;

        res->N=v1->N;
        res->storage=(float*)malloc(res->N*sizeof(float));        

        for (i=0; i<res->N; i++)
            res->storage[i]=v1->storage[i]+v2->storage[i];            

        return 1;
    }
    else
        return 0;
}


int dot(const Vector *v1, const Vector *v2, float *res)
{
    if (v1->N==v2->N)
    {
        int i;

        *res=0.0;

        for (i=0; i<v1->N; i++)
            *res+=v1->storage[i]*v2->storage[i];

        return 1;
    }
    else
        return 0;
}


int main()
{
    Vector v1, v2, w;
    float mul;

    v1.N=3;
    v2.N=3;

    // allocate memory on the heap for vectors components
    v1.storage=(float*)malloc(v1.N*sizeof(float));
    v2.storage=(float*)malloc(v2.N*sizeof(float));

    // fill up vectors components
    v1.storage[0]=1.0F;  v2.storage[0]=0.0F;
    v1.storage[1]=2.0F;  v2.storage[1]=1.0F;
    v1.storage[2]=3.0F;  v2.storage[2]=0.0F;

    fprintf(stdout,"first vector components\n");
    print_vector(&v1);

    fprintf(stdout,"second vector components\n");
    print_vector(&v2);    

    // execute algebric operations on vectors
    add(&v1,&v2,&w);
    dot(&v1,&v2,&mul);

    // dump results
    fprintf(stdout,"sum\n");
    print_vector(&w);

    fprintf(stdout,"dot product\n");
    fprintf(stdout,"%.3f\n",mul);

    // free up the heap
    free(v1.storage);
    free(v2.storage);
    free(w.storage);

    return 0;
}

